import { Route, Router } from "@solidjs/router";
import { listen } from "@tauri-apps/api/event";
import { createEffect, createSignal, onCleanup, Show } from "solid-js";
import { Login } from "./Login";
import { ProtectedRoute } from "./ProtectedRoute";
import { Search } from "./Search";
import { Settings } from "./Settings";
import { ImageViewer } from "./components/ImageViewer";
import type { sendImage } from "./network";
import { ImageStatus } from "./components/image-status/ImageStatus";
import { invoke, type PluginListener } from "@tauri-apps/api/core";
import { SearchImageContextProvider } from "./contexts/SearchImageContext";
import { platform } from "@tauri-apps/plugin-os";
import {
    listenForShareEvents,
    type ShareEvent,
} from "tauri-plugin-sharetarget-api";
import { readFile } from "@tauri-apps/plugin-fs";

const currentPlatform = platform();

export const App = () => {
    const [processingImage, setProcessingImage] =
        createSignal<Awaited<ReturnType<typeof sendImage>>>();

    const [file, setFile] = createSignal<File>();

    createEffect(() => {
        // TODO: Don't use window.location.href
        const unlisten = listen("focus-search", () => {
            window.location.href = "/";
        });

        onCleanup(() => {
            unlisten.then((fn) => fn());
        });
    });

    createEffect(() => {
        if (currentPlatform !== "ios" && currentPlatform !== "android") {
            console.log("not correct platform");
            return;
        }

        let listener: PluginListener;
        const setupListener = async () => {
            listener = await listenForShareEvents(
                async (intent: ShareEvent) => {
                    const contents = await readFile(intent.stream ?? "").catch(
                        (error: Error) => {
                            console.warn("fetching shared content failed:");
                            throw error;
                        },
                    );

                    console.log(intent);

                    setFile(
                        new File([contents], intent.name ?? "no-name", {
                            type: intent.content_type,
                        }),
                    );
                },
            );
        };
        setupListener();
        return () => {
            listener?.unregister();
        };
    });

    const onTakeScreenshot = () => {
        invoke("take_screenshot");
    };

    return (
        <SearchImageContextProvider>
            <Show when={currentPlatform === "linux"}>
                <button type="button" onClick={onTakeScreenshot}>
                    Take Screenshot [wayland :(]
                </button>
            </Show>
            <p>{file()?.name}</p>
            <ImageViewer onSendImage={setProcessingImage} />
            <ImageStatus
                processingImage={processingImage}
                onSetProcessingImage={setProcessingImage}
            />
            <Router>
                <Route path="/login" component={Login} />

                <Route path="/" component={ProtectedRoute}>
                    <Route path="/" component={Search} />
                    <Route path="/settings" component={Settings} />
                </Route>
            </Router>
        </SearchImageContextProvider>
    );
};
